var harness = new Siesta.Project.Browser.ExtJS()

harness.configure({
    title               : 'Siesta Examples',
    viewDOM             : true,

    enableCodeCoverage  : harness.isStandardPackage(), // Only in full version of Siesta
    coverageUnit        : 'extjs_class', // can be "file" or "extjs_class"

    // Define any global JS and CSS dependencies, these files will be injected into each test.
    preload             : [],

    // If your tests use dynamic loading, you can setup your paths using the 'loaderPath' config.
    loaderPath          : { 'Ext.ux' : '//cdn.sencha.com/ext/gpl/5.1.0/examples/ux' },

    recorderConfig      : {
        // enable/disable certain features of the recorder, see the Siesta.Recorder.Recorder documentation
        recordMouseMoveOnIdle   : true,
        recordPointsOfInterest  : true,
        recordMouseMovePath     : false,
        recordScroll            : false,
        recordInitialWindowSize : false
    }
});

harness.start(
    !harness.isStandardPackage() ? null : {
        group               : 'Native events',

        simulation          : 'native',

        items               : [
            {
                pageUrl     : '8.native-events/native.html',
                url         : '8.native-events/010-native.t.js'
            }
        ]
    },
    // Start here, and learn the basic unit testing functionality in Siesta
    {
        group               : 'Unit tests',
        autoCheckGlobals    : true,
        sandbox             : false, // speeds it up, sharing the same iframe for tests in this group

        items : [
            '1.unit-tests/basic_assertions.t.js',
            '1.unit-tests/async_code.t.js',
            '1.unit-tests/todo_tests.t.js',
            '1.unit-tests/wait_for.t.js',
            {
                // uncomment to "snooze" any failures from this test till 2050 :)
                //snooze          : '2050-01-01',
                url             : '1.unit-tests/bdd.t.js'
            },
            '1.unit-tests/spies.t.js',

            // A test descriptor can also be an object with `url` property and its own config options
            {
                autoCheckGlobals    : true,
                url                 : '1.unit-tests/global_variables.t.js',
                // change the title of the corresponding row in the test grid
                title               : 'global variables',
                // change the description (tooltip) of the row
                desc                : 'This test exercises detection of the global namespace pollution'
            },
            // Siesta can also run a Jasmine test suite
            {
                title           : 'Jasmine tests',
                jasmine         : true,
                expectedGlobals : [ 'Player', 'Song' ],
                // url should point to the specs runner html page in this case
                url             : '1.unit-tests/jasmine_suite/SpecRunner.html',
                // a link to some external resource, can be activated with CTRL+click on the title
                referenceUrl    : 'https://www.bryntum.com/docs/siesta/#!/api/Siesta.Project.Browser-cfg-jasmine',
                desc            : 'This test shows how existing Jasmine test suite can be run with Siesta'
            },
            harness.supportsArrowFunctions ? '1.unit-tests/returning_promise.t.js' : null,
            harness.supportsArrowFunctions ? '1.unit-tests/chain_step_with_promise.t.js' : null,
            harness.supportEcmaModules ? { url: '1.unit-tests/ecma-module.t.js', isEcmaModule: true } : null
        ]
    },

    {
        group   : 'UI tests',
        preload : [
            "//cdn.sencha.com/ext/gpl/5.1.0/packages/ext-theme-crisp/build/resources/ext-theme-crisp-all.css",
            "//cdn.sencha.com/ext/gpl/5.1.0/build/ext-all.js"
        ],
        items   : [
            {
                group : 'General',

                items : [
                    '2.ui-tests/general/resize.t.js',
                    '2.ui-tests/general/window.t.js',
                    '2.ui-tests/general/targeting_buttons.t.js',
                    '2.ui-tests/general/ext-bug.t.js',
                    {
                        runCore : 'sequential',
                        url     : '2.ui-tests/general/combo.t.js'
                    },
                    {
                        runCore : 'sequential',
                        url     : '2.ui-tests/general/custom-combo.t.js'
                    }
                ]
            },
            {
                group : 'Drag and drop',

                items : [
                    {
                        // Specify your own HTML page if you want
                        pageUrl : '2.ui-tests/drag-drop/cats.html',
                        url     : '2.ui-tests/drag-drop/drag-drop.t.js'
                    },
                    '2.ui-tests/drag-drop/dd-tree.t.js'
                ]
            },
            {
                group : 'Forms',

                items : [
                    '2.ui-tests/form/basic_form.t.js',
                    '2.ui-tests/form/checkboxes.t.js'
                ]
            },
            {
                group     : 'Grid',
                testClass : Your.Test.Class,

                items : [
                    '2.ui-tests/grid/basic.t.js',
                    '2.ui-tests/grid/waitfor_grid.t.js',
                    '2.ui-tests/grid/mouseover.t.js',
                    '2.ui-tests/grid/sel_model.t.js',
                    '2.ui-tests/grid/resizing.t.js',
                    // Tests that rely on browser focus should be run sequentially to prevent different
                    // tests stealing focus from each other
                    {
                        url     : '2.ui-tests/grid/editing.t.js',
                        runCore : 'sequential'
                    },
                    {
                        url     : '2.ui-tests/grid/editing_with_row_editor.t.js',
                        runCore : 'sequential'
                    }
                ]
            }
        ]
    },

    {
        group   : 'Application tests',
        // run tests one by one instead of several in parallel
        runCore : 'sequential',
        enableCodeCoverage : false,

        // uncomment to run the tests in popup
        //runInPopup  : true,
        items   : [
            {
                title           : 'Basic Ext JS MVC app',
                pageUrl         : '3.application-tests/extjs-mvc/app.html',
                url             : '3.application-tests/extjs-mvc/tests/app.t.js'
            },
            {
                title           : 'Ext JS Ticket app',
                pageUrl         : '3.application-tests/ticket-app/',
                url             : '3.application-tests/ticket-app/tests/smoke_test.t.js'
            },
            {
                title           : 'Responsive app test - landscape',
                viewportWidth   : 1024,
                viewportHeight  : 768,
                pageUrl         : '3.application-tests/executive-dashboard/',
                url             : '3.application-tests/executive-dashboard/tests/large-size.t.js'
            },
            {
                title           : 'Responsive app test - portrait',
                viewportWidth   : 500,
                viewportHeight  : 700,
                pageUrl         : '3.application-tests/executive-dashboard/',
                url             : '3.application-tests/executive-dashboard/tests/small-size.t.js'
            },
            {
                title           : 'Monkey testing',
                viewportWidth   : 1024,
                viewportHeight  : 768,
                pageUrl         : '3.application-tests/executive-dashboard/',
                url             : '3.application-tests/executive-dashboard/tests/monkey_test.t.js'
            },
            // do not show this example in Lite version
            !harness.isStandardPackage() ? null :
            {
                // make sure we'll reach the correct exit point
                title           : 'Page redirect',
                needDone        : true,
                enablePageRedirect : true,
                pageUrl         : '3.application-tests/page-redirect/source_page.html',
                url             : '3.application-tests/page-redirect/page_redirect.t.js'
            }
        ]
    },

    /**
     * Siesta can test any JavaScript or web page, here are some samples of other JS frameworks
     */
    {
        group : '3rd party JS-libraries',
        items : [
            {
                group       : 'Angular',
                preload     : [
                    '4.testing-3rd-party-libraries/angularjs/angular.min.js',
                    '4.testing-3rd-party-libraries/angularjs/angular-mocks.js',
                    '4.testing-3rd-party-libraries/angularjs/app.js'
                ],
                items       : [
                    '4.testing-3rd-party-libraries/angularjs/angular.t.js'
                ]
            },
            {
                group     : 'jQuery',
                testClass : Siesta.Test.jQuery,

                preload : [
                    // Jquery CDN
                    'http://ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min.js'
                ],

                items : [
                    '4.testing-3rd-party-libraries/jquery/hello_world.t.js',
                    {
                        pageUrl     : '4.testing-3rd-party-libraries/jquery/jquery_ui.html',
                        preload     : 'inherit',
                        alsoPreload : [
                            'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/jquery-ui.min.js',
                            'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/themes/black-tie/jquery-ui.css'
                        ],
                        url         : '4.testing-3rd-party-libraries/jquery/jquery_ui_selectable.t.js'
                    },
                    {
                        url         : '4.testing-3rd-party-libraries/jquery/monkey.t.js'
                    }
                ]
            },
            // polymer example throws exception in IE9 - this is not Siesta's fault, so we just hide it in IE9
            bowser.chrome || bowser.safari || bowser.msie && bowser.version <= 9 ? null :
            {
                group       : 'Polymer',
                pageUrl     : '4.testing-3rd-party-libraries/polymer/finished',
                items       : [
                    '4.testing-3rd-party-libraries/polymer/polymer.t.js'
                ]
            },
            {
                group     : 'PrototypeJS',
                testClass : Siesta.Test.Browser,
                preload   : [
                    "4.testing-3rd-party-libraries/prototypejs/prototype.js"
                ],
                items     : [
                    '4.testing-3rd-party-libraries/prototypejs/hello_world.t.js'
                ]
            },
            // react example throws exception in IE < 10 - this is not Siesta's fault, so we just hide it in such case
            $.browser.msie && $.browser.version <= 10 ? null :
            {
                group       : 'React',
                pageUrl     : '4.testing-3rd-party-libraries/react/',
                items       : [
                    '4.testing-3rd-party-libraries/react/react.t.js'
                ]
            }
        ]
    },
    !harness.isStandardPackage() ? null : {
        group           : 'Code coverage',
        referenceUrl    : 'https://www.bryntum.com/docs/siesta/#!/guide/code_coverage',
        preload : [
            "//cdn.sencha.com/ext/gpl/4.2.1/resources/css/ext-all.css",
            "//cdn.sencha.com/ext/gpl/4.2.1/ext-all-debug.js",
            {
                url        : "5.code_coverage/several_classes_in_one_file.js",
                instrument : true
            },
            {
                url        : "5.code_coverage/some_folder/preload_file.js",
                instrument : true
            },
            {
                url        : "5.code_coverage/some_folder/folder2/preload_file.js",
                instrument : true
            }
        ],
        items   : [
            {
                group : 'Static loading',

                items : [
                    '5.code_coverage/010_range.t.js',
                    '5.code_coverage/020_event.t.js'
                ]
            },
            {
                group : 'Dynamic loading',

                loaderPath : {
                    My : '5.code_coverage/lib/My'
                },

                preload : [
                    "//cdn.sencha.com/ext/gpl/4.2.1/resources/css/ext-all.css",
                    "//cdn.sencha.com/ext/gpl/4.2.1/ext-all-debug.js",
                    {
                        url        : "5.code_coverage/several_classes_in_one_file.js",
                        instrument : true
                    }
                ],

                items : [
                    '5.code_coverage/030_loader.t.js'
                ]
            },
            {
                // example of getting code coverage for application
                group : 'Code coverage for application code',

                pageUrl : '5.code_coverage/pageUrl.html',

                items : [
                    '5.code_coverage/040_pageUrl.t.js'
                ]
            }
        ]
    },
    {
        group   : 'Misc',

        // loader instrumentation hook switches ExtJS loader to synchronous loading mode (using XHR)
        // and fetching SimManager will fail
        installLoaderInstrumentationHook    : false,

        preload : [
            "//cdn.sencha.com/ext/gpl/5.1.0/packages/ext-theme-crisp/build/resources/ext-theme-crisp-all.css",
            "//cdn.sencha.com/ext/gpl/5.1.0/build/ext-all.js"
        ],
        items   : [
            '6.misc/ajax.t.js',
            {
                requires : [ 'Ext.ux.ajax.SimManager' ],
                url      : '6.misc/ajax-mock.t.js'
            },
            '6.misc/native-dialogs.t.js',
            {
                simulation  : 'synthetic',
                url         : '6.misc/popups.t.js'
            }
        ]
    }
);
// eof Harness.start
