/*

Siesta 4.4.5
Copyright(c) 2009-2018 Bryntum AB
https://bryntum.com/contact
https://bryntum.com/products/siesta/license

*/
Class('Siesta.Launcher.NodeJS', {

    isa         : Siesta.Launcher.BaseLauncher,

    does        : [
        Siesta.Launcher.CommandLineTool.NodeJSTool
    ],

    has : {
        executableName          : 'nodejs',

        sharedNativeSimulator   : false,

        knownOptionGroups   : {
            init : {
                '15-nodejs' : {
                    name        : 'NodeJS options'
                }
            }
        },

        knownOptions            : {
            init : [
                {
                    name    : 'isolate-console',
                    desc    : [
                        'This option disables the "shared" console - the `stdin/stdout/stderr` streams of all test files will not be forwarded',
                        'to the main console.'
                    ],
                    group   : '20-misc'
                },
                {
                    name    : 'node-arg',
                    desc    : [
                        'This option can be repeated several times, and specifies command line arguments for the derived NodeJS process',
                        'in the form --node-arg name=value, for example:\n',
                        '    --node-arg=\'--inspect-port=12345\' --node-arg=\'--no-deprecation\'\n',
                        'Full options list is listed in: > node --help\n',
                        'NOTE, that by default, all unknown options are translated to the derived NodeJS process'
                    ],
                    group   : '20-misc'
                }
            ]
        },

        helpIntro               : function () {
            var style   = this.style()

            return [
                style.bold('Usage: ') + this.executableName + ' [glob] [OPTIONS]\n',
                'Will launch all tests matching ' + style.bold('glob') + ' (default value is "**/*.t.?(m)js").',
                'Use trailing ' + style.bold('/') + ' to match directories.',
                'All options are optional.',
                ''
            ]
        }
    },


    methods : {

        createOptionsWrapper : function (cfg) {
            return new Siesta.Launcher.Options.NodeJS({ options : cfg, launcher : this })
        },


        onUnknownOption : function (value, name) {
            var str     = this.optionToString(value, name)

            this.info("Unknown option forwarded to child NodeJS process: " + str)

            var nodeJsArgs      = this.options[ 'node-arg' ] = this.options[ 'node-arg' ] || []
            if (!(nodeJsArgs instanceof Array)) nodeJsArgs = [ nodeJsArgs ]

            nodeJsArgs.push(str)
        },


        // this method runs before the "prepareOptions" call
        createRunners : function () {
            return [
                new Siesta.Launcher.Runner.NodeJS({
                    launcher    : this
                })
            ]
        },


        printVersion : function () {
            this.SUPERARG(arguments)

            this.print(this.style().bold("NodeJS    : ") + process.version)
        }
    }
    // eof methods
})
