/*

Siesta 4.4.5
Copyright(c) 2009-2018 Bryntum AB
https://bryntum.com/contact
https://bryntum.com/products/siesta/license

*/
!function () {

var fs              = require('fs')
var fse             = require('fs-extra')

Class('Siesta.Launcher.Page.Puppeteer', {

    isa         : Siesta.Launcher.Page.BasePage,

    does        : [
        Siesta.Launcher.Page.CanWorkWithImagesNodeJS
    ],

    has : {
        puppeteerPage       : { required : true }
    },


    methods : {

        screenshot : function (command) {
            var me              = this
            var options         = this.dispatcher.options
            var launcher        = this.launcher
            var baseFileName    = command.fileName.replace(/(\.png)?$/, '.png')
            var baseDir         = options[ 'screenshot-dir' ]
            var path            = require('path')

            var fileName        = launcher.normalizePath(baseDir ? path.join(baseDir, baseFileName) : baseFileName)

            if (!launcher.ensurePathExists(fileName)) {
                return Promise.resolve({ success : false, error : "Can't create target directory for file: " + fileName })
            }

            var tempFile        = require('tmp').fileSync({ prefix : 'ssi', postfix : '.png' }).name

            return this.puppeteerPage.screenshot({
                path        : tempFile,
                type        : 'png',
                clip        : {
                    x           : command.x,
                    y           : command.y,
                    width       : command.width,
                    height      : command.height
                }
            }).then(function () {
                var compareWithPrevious = Boolean(options[ 'screenshot-compare-with-previous' ])
                var compareWithBase     = options[ 'screenshot-compare-with-base' ]

                if (!command.skipCompare && (compareWithPrevious && launcher.fileExists(fileName) || compareWithBase)) {
                    var compareToFile

                    if (compareWithPrevious) {
                        compareToFile   = fileName.replace(/\.png$/, '-prev.png')

                        if (launcher.fileExists(compareToFile)) launcher.deleteFile(compareToFile)

                        fs.renameSync(fileName, compareToFile)
                    }

                    fse.copySync(tempFile, fileName, { clobber : true })

                    if (compareWithBase) {
                        compareToFile   = path.join(compareWithBase, baseFileName)

                        if (!launcher.fileExists(compareToFile)) return {
                            success     : false,
                            error       : 'Base file `' + path.resolve(compareToFile) + '` does not exists'
                        }
                    }

                    var diffFileName    = fileName.replace(/\.png$/, '-diff.png')

                    return me.compareImages(
                        path.resolve(fileName), path.resolve(compareToFile), path.resolve(diffFileName), command.cfg
                    ).then(function (res) {
                        return res && res.exitCode != 2 ? {
                            success         : true,
                            compare         : compareWithPrevious ? 'previous' : 'base',
                            similar         : res.similar,
                            similarity      : res.similarity,
                            newScreenshot   : path.resolve(fileName),
                            oldScreenshot   : path.resolve(compareToFile)
                        } : {
                            success     : false,
                            error       : 'Error while comparing the images: ' + res.output
                        }
                    })

                } else {
                    fse.copySync(tempFile, fileName, { clobber : true })

                    return { success : true, compare : false }
                }
            })
        },


        open : function (url, callback) {
            var page    = this.puppeteerPage

            page.setViewport({
                width       : this.width,
                height      : this.height
            }).then(function () {

                page.goto(url).then(function () {
                    callback()
                }, function () {
                    callback("Can't open page: " + url)
                })
            })
        },


        close : function () {
            var puppeteerPage   = this.puppeteerPage

            // already closed
            if (!puppeteerPage) return

            var me              = this

            this.puppeteerPage  = null

            var promise         = puppeteerPage.close().catch(function (e) {
                me.debug("Exception during `puppeteerPage.close()`: " + e)
            })

            return Promise.all([ promise, this.SUPER() ])
        },


        runChunk : function (chunkTask, callback) {
            var me          = this
            var SUPER       = this.SUPER

            if (me.dispatcher.hasNativeSimulation) {
                var simulatorServerClass = require("../src/simulator/puppeteer/SimulatorServerPuppeteer").SimulatorServerPuppeteer

                me.setupNativeEventsSimulator(simulatorServerClass).then(
                    function () {
                        me.wsServer.simulatorServerConfig = { page : me.puppeteerPage }

                        SUPER.call(me, chunkTask, callback)
                    },
                    function (e) {
                        callback(e)
                    }
                )
            } else
                SUPER.call(me, chunkTask, callback)
        },


        executeSmallScriptPromised : function (text, ignoreException) {
            var me          = this

            return new Promise(function (resolve, reject) {
                var func    = eval('(function() {' + text + '})')

                me.puppeteerPage.evaluate(func).then(function (res) {
                    resolve(res)

                }, function (e) {
                    if (!ignoreException) {
                        me.print("<Exception from launcher>")
                        me.print("    While running small script: " + text.substring(0, 150))
                        me.print("    Exception: " + e)
                        me.print("</Exception from launcher>")

                        reject(e)

                        return
                    }

                    resolve(null)
                })
            })
        }
    }
    // eof methods
})


}();