/*

Siesta 4.4.5
Copyright(c) 2009-2018 Bryntum AB
https://bryntum.com/contact
https://bryntum.com/products/siesta/license

*/
!function () {

var fs              = require('fs')
var fse             = require('fs-extra')
var child_process   = require('child_process');

Class('Siesta.Launcher.Page.WebDriverNodeJS', {

    isa         : Siesta.Launcher.Page.BasePage,

    does        : Siesta.Launcher.Page.CanWorkWithImagesNodeJS,

    has : {
        driver                  : { required : true },

        xvfbLockResult          : null
    },


    methods : {

        buildSessionId : function () {
            return String(this.driver.session_.value_.getId())
        },


        pageShouldNotBeUsedAfterException : function (e) {
            return /Request timeout/.test(e)
        },

        // promised method
        screenshot : function (command) {
            var me              = this
            var options         = this.dispatcher.options
            var launcher        = this.launcher
            var baseFileName    = command.fileName.replace(/(\.png)?$/, '.png')
            var baseDir         = options[ 'screenshot-dir' ]
            var path            = require('path')

            var fileName        = launcher.normalizePath(baseDir ? path.join(baseDir, baseFileName) : baseFileName)

            if (!launcher.ensurePathExists(fileName)) {
                return Promise.resolve({ success : false, error : "Can't create target directory for file: " + fileName })
            }

            var tempFile        = require('tmp').fileSync({ prefix : 'ssi' }).name

            return this.driver.takeScreenshot().then(function (screenshot) {
                fs.writeFileSync(tempFile, screenshot, 'base64')

                return me.getPNGImageSize(tempFile)
            }).then(function (size) {
                var k = 1

                return me.cropImage(
                    tempFile,
                    command.x * k,
                    command.y * k,
                    Math.min(size.width - command.x * k, command.width * k),
                    Math.min(size.height - command.y * k, command.height * k)
                )
            }).then(function (croppedFileName) {
                var compareWithPrevious = Boolean(options[ 'screenshot-compare-with-previous' ])
                var compareWithBase     = options[ 'screenshot-compare-with-base' ]

                if (!command.skipCompare && (compareWithPrevious && launcher.fileExists(fileName) || compareWithBase)) {
                    var compareToFile

                    if (compareWithPrevious) {
                        compareToFile   = fileName.replace(/\.png$/, '-prev.png')

                        if (launcher.fileExists(compareToFile)) launcher.deleteFile(compareToFile)

                        fs.renameSync(fileName, compareToFile)
                    }

                    fse.copySync(croppedFileName, fileName, { clobber : true })

                    if (compareWithBase) {
                        compareToFile   = path.join(compareWithBase, baseFileName)

                        if (!launcher.fileExists(compareToFile)) return {
                            success     : false,
                            error       : 'Base file `' + path.resolve(compareToFile) + '` does not exists'
                        }
                    }

                    var diffFileName    = fileName.replace(/\.png$/, '-diff.png')

                    return me.compareImages(
                        path.resolve(fileName), path.resolve(compareToFile), path.resolve(diffFileName), command.cfg
                    ).then(function (res) {
                        return res && res.exitCode != 2 ? {
                            success         : true,
                            compare         : compareWithPrevious ? 'previous' : 'base',
                            similar         : res.similar,
                            similarity      : res.similarity,
                            newScreenshot   : path.resolve(fileName),
                            oldScreenshot   : path.resolve(compareToFile)
                        } : {
                            success     : false,
                            error       : 'Error while comparing the images: ' + res.output
                        }
                    })

                } else {
                    fse.copySync(croppedFileName, fileName, { clobber : true })

                    return { success : true, compare : false }
                }
            })
        },


        shouldStopScriptExecution : function () {
            return SHUTDOWN
        },


        open : function (url, callback) {
            var me          = this
            var driver      = me.driver

            driver.get(url).then(function () {
                // setPosition also updates window size (on MacOS) for some reason, so, on MacOS it must be done before setSize
                // on other platforms its the opposite
                return me.launcher.isMacOS ? driver.manage().window().setPosition(1, 1) : Promise.resolve()

            }).then(function () {

                return driver.manage().window().setSize(me.width - 1, me.height - 1)

            }).then(function () {

                return !me.launcher.isMacOS ? driver.manage().window().setPosition(1, 1) : Promise.resolve()

            }).then(function () {
                callback()
            }, function (e) {
                callback(e)
            })
        },


        runChunk : function (chunkTask, callback) {
            var me          = this
            var SUPER       = this.SUPER

            if (!me.getOptions().host && me.dispatcher.hasNativeSimulation && !me.launcher.sharedNativeSimulator) {
                // this is basically Xvfb branch

                me.setupNativeEventsSimulatorInSeparateProcess(me.xvfbLockResult.displayNum).then(
                    function () {
                        SUPER.call(me, chunkTask, callback)
                    },
                    function (e) {
                        callback(e)
                    }
                )
            } else
                SUPER.call(me, chunkTask, callback)
        },


        close : function () {
            // already closed
            if (!this.driver) return Promise.resolve()

            if (this.xvfbLockResult) {
                this.xvfbLockResult.release()
                this.xvfbLockResult = null
            }

            // IE driver as anything related to IE sporadically throws exceptions when closing the browser..
            try {
                // seems webdriver queues the "driver.quit()" calls and subsequent calls to page creation are delayed
                // so trying to avoid that
                var promise = Promise.resolve(this.driver.quit()).then(function () {
                    return Promise.resolve()
                }, function () {
                    return Promise.resolve()
                })
            } catch (e) {
                this.printError("Sync error while closing page: " + e)
            }

            this.driver     = null

            return Promise.all([ promise || Promise.resolve(), this.SUPER() ])
        },


        // promised method
        executeSmallScriptPromised : function (text, ignoreException) {
            var me          = this

            if (this.shouldStopScriptExecution()) return Promise.resolve(null)

            return Promise.resolve(this.driver.executeScript(text))[ 'catch' ](function (e) {
                // IE driver as everything related to IE sporadically throws exceptions when executing the code
                // it also throws exception when trying to execute the script on 404 page (all other drivers are ok)
                if (!ignoreException && !me.shouldStopScriptExecution()) {
                    me.debug("<Exception from launcher, pageId=" + me.id + ", sessionId=" + me.getSessionId() + ">")
                    me.debug("    While running small script: " + text.substring(0, 250))
                    me.debug("    Exception: " + e)
                    me.debug("</Exception from launcher>")

                    throw e
                } else
                    return null
            })
        }
    }
    // eof methods
})

}();