/*

Siesta 4.4.5
Copyright(c) 2009-2018 Bryntum AB
https://bryntum.com/contact
https://bryntum.com/products/siesta/license

*/
Class('Siesta.Launcher.Puppeteer', {

    isa         : Siesta.Launcher.BaseLauncher,

    does        : [
        Siesta.Launcher.CommandLineTool.NodeJSTool
    ],

    has : {
        executableName          : 'puppeteer',

        sharedNativeSimulator   : false,

        knownOptionGroups   : {
            init : {
                '15-screenshot' : {
                    name        : 'Screenshots options'
                }
            }
        },

        knownOptions            : {
            init : [
                {
                    name    : 'headless',
                    desc    : [
                        'Default value is `true`, which opens Chrome in headless mode. Set to `false` to see the browser window.'
                    ],
                    // put at the top
                    group   : '00-system'
                },
                {
                    name    : 'simulation',
                    desc    : [
                        'Setting this option to `native` from the command line will override the `simulation` config on the project level. ',
                        'This means that all tests/test groups that don\'t explicitly specify this config, will use the value from the',
                        'command line'
                    ],
                    group   : '10-basic'
                },
                // COPIED from Siesta.Launcher.WebDriverNodeJS, should be kept in sync
                {
                    name    : 'screenshot-dir',
                    desc    : [
                        'Specifies the base directory for the screenshots. Screenshot paths will be resolved',
                        'relatively this directory. Current working directory by default.'
                    ],
                    group   : '15-screenshot'
                },
                {
                    name    : 'screenshot-compare-with-previous',
                    desc    : [
                        'If specified, screenshots will be compared with the currently existing files.',
                        'If screenshots will be found to be different, a failing assertion will be added to the test and',
                        'the diff image file will be created, having a name of the screenshot image, suffixed with the `-diff`',
                        'The previous screenshot file will be kept with the `-prev` suffix in the file name.',
                        'See also the project option `screenshotCompareConfig`'
                    ],
                    group   : '15-screenshot'
                },
                {
                    name    : 'screenshot-compare-with-base',
                    desc    : [
                        'A base directory of the other set of screenshots. Screenshots will be compared',
                        'with the correspoding screenshot from that base. For example, if `--screenshot-dir`',
                        'option is set to `chrome`, `--screenshot-compare-with-base` to `firefox`',
                        'and screenshot`s file name is `grids/users.png`, then the following screenshots',
                        'will be compared: `chrome/grids/users.png` (new version)',
                        'and `firefox/grids/users.png` (base version).',
                        'If screenshots will be found to be different, a failing assertion will be added to the test and',
                        'the diff image file will be created, having a name of the screenshot image, suffixed with the `-diff`:',
                        '`chrome/grids/users-diff.png`.',
                        'See also the project option `screenshotCompareConfig`'
                    ],
                    group   : '15-screenshot'
                },
                // EOF COPIED
                // COPIED from Siesta.Launcher.WebDriverNodeJS, slightly changed
                {
                    name    : 'browser-arg',
                    desc    : [
                        'This option can be repeated several times, and specifies command line arguments for the browser process',
                        'in the form --browser-arg name=value, for example:\n',
                        '    --browser-arg multiple-automatic-downloads=1 --browser-arg disable-gpu\n',
                        'Full options list can be found here: https://peter.sh/experiments/chromium-command-line-switches/'
                    ],
                    group   : '20-misc'
                }
                // EOF COPIED
            ]
        }
    },


    methods : {

        createOptionsWrapper : function (cfg) {
            return new Siesta.Launcher.Options.Puppeteer({ options : cfg, launcher : this })
        },


        // this method runs before the "prepareOptions" call
        createRunners : function () {
            return [
                new Siesta.Launcher.Runner.Puppeteer({
                    launcher    : this
                })
            ]
        },


        buildHarnessUrlQueryParams : function () {
            return Joose.O.extend(this.SUPER(), {
                puppeteer       : true
            })
        },


        printVersion : function () {
            this.SUPERARG(arguments)

            this.print("NodeJS    : " + process.version)
            this.print("Puppeteer : " + require('puppeteer/package.json').version)
        }
    }
    // eof methods
})
