import {Base, Constructable, Mixin} from "../Common.js";

let vm      = require('vm')

export const CanRequireInContext = <T extends Constructable>(base : T) => class CanRequireInContext extends base {

    requireInContext (fileName : string, ctx? : object, requireFun? : Function) : object {
        let old             = vm.runInThisContext

        let context : any   = ctx

        if (!ctx) {
            context = Object.assign(ctx || {}, {
                Buffer          : Buffer,
                URL             : URL,
                URLSearchParams : URLSearchParams,

                process         : process,
                console         : console,

                setImmediate    : setImmediate,
                clearImmediate  : clearImmediate,
                setTimeout      : setTimeout,
                clearTimeout    : clearTimeout,
                setInterval     : setInterval,
                clearInterval   : clearInterval
            })

            context.global      = context

            vm.createContext(context)
        }

        vm.runInThisContext = (code, options) => {
            return vm.runInContext(code, context, options)
        }

        (requireFun || require)(fileName)

        vm.runInThisContext = old

        return context
    }


    runFuncInContext (func : Function, context : object, options)  {
        return vm.runInContext(';(' + func.toString() + ')()', context, options)
    }

}

export type CanRequireInContext = Mixin<typeof CanRequireInContext>

export const requirer = new (CanRequireInContext(Base))()