Ext.data.JsonP.getting_started_sencha({"guide":"<h2 id='getting_started_sencha-section-introduction'>Introduction</h2>\n<div class='toc'>\n<p><strong>Contents</strong></p>\n<ol>\n<li><a href='#!/guide/getting_started_sencha-section-introduction'>Introduction</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-siesta-browser-project'>Siesta browser project</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-sandboxing.-test-page-vs-project-web-interface-page'>Sandboxing. Test page vs project web-interface page</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-configuring-the-project'>Configuring the project</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-configuring-the-individual-test'>Configuring the individual test</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-simulating-user-interaction'>Simulating user interaction</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-user-actions-recorder'>User actions recorder</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-limitations-'>Limitations </a></li>\n<li><a href='#!/guide/getting_started_sencha-section-writing-tests-in-bdd-style'>Writing tests in BDD style</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-assertions'>Assertions</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-user-interface'>User interface</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-testing-asynchronous-code'>Testing asynchronous code</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-detecting-global-variable-leaks'>Detecting global variable leaks</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-%22todo%22-assertions'>\"TODO\" assertions</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-extending-the-test-class'>Extending the test class</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-automation'>Automation</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-best-practices'>Best practices</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-advanced-setup'>Advanced setup</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-running-tests-in-nodejs'>Running tests in NodeJS</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-buy-this-product'>Buy this product</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-support'>Support</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-see-also'>See also</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-attribution'>Attribution</a></li>\n<li><a href='#!/guide/getting_started_sencha-section-copyright-and-license'>COPYRIGHT AND LICENSE</a></li>\n</ol>\n</div>\n\n<p>Siesta is a stress-free JavaScript unit and UI-testing tool. It is known that change of activity is a form of rest,\nso stop writing code, write some tests and have some rest! Your application will win from both.</p>\n\n<p>Siesta is very easy to learn and as your test suite grows and your requirements becomes more complex it still scales very well.</p>\n\n<p>Siesta is also cross-platform - the same tests can run in browsers and NodeJS, on Linux, MacOs and Windows\n(assuming of course they are written in a platform-independent manner). Headless browser modes are supported.</p>\n\n<p><p><img src=\"guides/getting_started_sencha/images/demo.png\" alt=\"autogen\" width=\"1600\" height=\"1200\"></p></p>\n\n<p>In this guide, we assume a setup, pointed toward running tests in browsers.</p>\n\n<p>For the setup, targeting NodeJS environment, please refer to this guide <a href=\"#!/guide/bdd_conventions\">Siesta getting started with NodeJS.</a></p>\n\n<h2 id='getting_started_sencha-section-siesta-browser-project'>Siesta browser project</h2>\n\n<p>Since plain browsers can't access the file system, we need to start with creation of the Siesta project - a single place for storing the meta-information\nabout your test suite.</p>\n\n<pre><code>const project = new <a href=\"#!/api/Siesta.Project.Browser\" rel=\"Siesta.Project.Browser\" class=\"docClass\">Siesta.Project.Browser</a>();\n\nproject.configure({\n    title       : 'Basic browser test suite',\n\n    preload     : [\n        'https://code.jquery.com/jquery-1.6.4.min.js'\n    ]\n});\n\nproject.plan(\n    '010_sanity.t.js',\n    '020_basic.t.js'\n);\n\nproject.start();\n</code></pre>\n\n<p>Lets save this file as <code>siesta.js</code> in the <code>tests</code> folder of your app. We'll need a wrapping html file for this project too, to be able to access\nthe web interface, lets name it <code>siesta.html</code>:</p>\n\n<pre><code>&lt;!DOCTYPE html&gt;\n&lt;html&gt;\n    &lt;head&gt;\n        &lt;!-- Recommended set of pragmas, required for IE11 compatibility--&gt;\n        &lt;meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\"&gt;\n        &lt;meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\"&gt;\n        &lt;meta name=\"viewport\" content=\"width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no\"&gt;\n\n        &lt;!-- Web interface --&gt;\n        &lt;link rel=\"stylesheet\" type=\"text/css\" href=\"__SIESTA_FOLDER__/resources/css/siesta-all.css\"&gt;\n        &lt;script type=\"text/javascript\" src=\"__SIESTA_FOLDER__/siesta-all.js\"&gt;&lt;/script&gt;\n\n        &lt;!-- Project file --&gt;\n        &lt;script type=\"text/javascript\" src=\"siesta.js\"&gt;&lt;/script&gt;\n    &lt;/head&gt;\n\n    &lt;body&gt;\n    &lt;/body&gt;\n&lt;/html&gt;\n</code></pre>\n\n<p>As you've probably already guessed, we've created a project with 2 test files: <code>010_sanity.t.js</code> and <code>020_basic.t.js</code>. <strong>The paths given, are relative\nto the <code>siesta.html</code> wrapper file</strong></p>\n\n<p>Lets check how the test file look itself:</p>\n\n<pre><code>StartTest(t =&gt; {\n\n    t.it(\"Sanity\", t =&gt; {\n\n        t.ok($, 'jQuery is here');\n\n        t.is(1, \"1\", \"Relaxed equality\")\n\n        t.exact(1, 1, \"Exact equality\")\n\n        t.is(1, t.anyNumberApprox(0.95, 0.1), \"Fuzzy equality\")\n\n        t.like(window.location.href, /010_sanity\\.t\\.js/, \"Location of the test file is as expected\")\n\n        t.throwsOk(\n            () =&gt; { vixie },\n            /vixie/,\n            \"Correct exception thrown\"\n        )\n    });\n})    \n</code></pre>\n\n<p>The test code is wrapped with <code>StartTest(t =&gt; { ... })</code> construct. The function passed to it will receive an instance\nof the <a href=\"#!/api/Siesta.Test\" rel=\"Siesta.Test\" class=\"docClass\">Siesta.Test</a> class, as the 1st argument.</p>\n\n<p>The instance of the test class has various <em>assertion</em> which are just methods of the class. An \"assertion\" is any statement about your code.\nIt may have arbitrary meaning, ranging from very simple like \"this variable is equal to that variable\" to complex and domain specific:\n\"this instance of Observable will fire this event exactly N times during the following X milliseconds\".</p>\n\n<p>Siesta has many general-purpose assertions built-in and it also allows you to <a href=\"#!/guide/71_extending_test_class\">create your own assertions.</a>\nThe statement of an assertion can be either truthy (we say \"assertion pass\") or false (assertion fail).</p>\n\n<p>In the example above we use the simplest possible assertion: <code>t.ok(value, description)</code>. It passes when the provided <code>value</code> is \"truthy\",\nlike : <code>true / 1 / \"TRUE\" / {}</code> etc and fails otherwise. Each assertion also has an optional description which should contain\nan explanation of what's being asserted.</p>\n\n<p>We also used <code>throwsOk</code> assertion, which, accepts a function, run it, and verify that it does throw exception, which serializes to the string,\nmatching a given <code>RegExp</code>. You've probably get the idea.</p>\n\n<p><strong>To see the list of all generic assertions, supported by Siesta</strong>, please refer to: <a href=\"#!/api/Siesta.Test\" rel=\"Siesta.Test\" class=\"docClass\">Siesta.Test</a> class.</p>\n\n<p><strong>For the list of browser-specific assertions</strong>, please refer to: <a href=\"#!/api/Siesta.Test.Browser\" rel=\"Siesta.Test.Browser\" class=\"docClass\">Siesta.Test.Browser</a> class.</p>\n\n<p>Ok, we now have a project file, a web-interface wrapper for it, and test file. Lets open the <code>siesta.html</code> file in a browser.</p>\n\n<p class=\"side-note\">\nWhile it is possible to use file-system only setup, using `file://` urls, it is not recommended, as support for it varies per browser\nand some features may be different from normal `http://` setup. \n<br><br>\nThe recommended way is to use a local web server on your machine. For a simple, zero-configuration one, just to serve the static files, you can check\nthis Node package: <a href=\"https://www.npmjs.com/package/local-web-server\">https://www.npmjs.com/package/local-web-server</a>\n<br><br>\nSo, the browser url should look like: http://localhost/yourproject/tests\n</p>\n\n\n<p>You should see something like this:</p>\n\n<p><p><img src=\"guides/getting_started_sencha/images/synopsys.png\" alt=\"autogen\" width=\"1600\" height=\"1200\"></p></p>\n\n<p>Click the \"Run all\" button in the toolbar. Notice, how the <code>010_sanity.t.js</code> test gains a green checkmark, indicating it has completed\nwithout failing assertions.</p>\n\n<p>The <code>020_basic.t.js</code> has a red cross indicating the file is missing - feel free to create it after reading a couple of the following sections :)</p>\n\n<h2 id='getting_started_sencha-section-sandboxing.-test-page-vs-project-web-interface-page'>Sandboxing. Test page vs project web-interface page</h2>\n\n<p>By default, <a href=\"#!/api/Siesta.Project.Browser-cfg-sandbox\" rel=\"Siesta.Project.Browser-cfg-sandbox\" class=\"docClass\">sandboxing</a> is enabled. Every test is launched in isolated browser JavaScript context. Isolation is\ndone using iframes or <a href=\"#!/api/Siesta.Project.Browser-cfg-runInPopup\" rel=\"Siesta.Project.Browser-cfg-runInPopup\" class=\"docClass\">popups</a>. In general, there is no need to cleanup anything after the test.</p>\n\n<p>This behavior is configurable with the <a href=\"#!/api/Siesta.Project.Browser-cfg-sandbox\" rel=\"Siesta.Project.Browser-cfg-sandbox\" class=\"docClass\">sandbox</a> option, but we recommend you keep it enabled\nuntil you start feeling comfortable with Siesta.</p>\n\n<p>One need to understand the difference between the test page - the iframe in which the test will be launched, and the project page - the web-interface\npage. These are not related, and run different code.</p>\n\n<p>Typical mistake is to include the files from your application to the web-interface page:</p>\n\n<pre><code>&lt;!-- Web interface --&gt;\n&lt;link rel=\"stylesheet\" type=\"text/css\" href=\"__SIESTA_FOLDER__/resources/css/siesta-all.css\"&gt;\n&lt;script type=\"text/javascript\" src=\"__SIESTA_FOLDER__/siesta-all.js\"&gt;&lt;/script&gt;\n\n&lt;!-- WRONG, web-interface does not need files from your projects --&gt;\n&lt;link rel=\"stylesheet\" type=\"text/css\" href=\"my/project/styles.css\"&gt;\n&lt;script type=\"text/javascript\" src=\"my/project/bundle.js\"&gt;&lt;/script&gt;\n\n&lt;!-- Project file --&gt;\n&lt;script type=\"text/javascript\" src=\"siesta.js\"&gt;&lt;/script&gt;\n</code></pre>\n\n<p>In the same manner don't include the Siesta files in the <a href=\"#!/api/Siesta.Project.Browser-cfg-preload\" rel=\"Siesta.Project.Browser-cfg-preload\" class=\"docClass\">preload</a> option:</p>\n\n<pre><code>project.configure({\n    title       : 'Awesome Test Suite',\n\n    preload     : [\n        '../../siesta-all.js', // WRONG, no need to include siesta files on the test page\n        '../my-app-all.js'\n    ]\n});\n</code></pre>\n\n<h2 id='getting_started_sencha-section-configuring-the-project'>Configuring the project</h2>\n\n<p>A project can be configured with the <a href=\"#!/api/Siesta.Project.Browser-method-configure\" rel=\"Siesta.Project.Browser-method-configure\" class=\"docClass\">configure</a> method, which can be called several times.\nSee the <a href=\"#!/api/Siesta.Project.Browser\" rel=\"Siesta.Project.Browser\" class=\"docClass\">Siesta.Project.Browser</a> for a detailed description of all available options.\nThe most important option is <a href=\"#!/api/Siesta.Project.Browser-cfg-preload\" rel=\"Siesta.Project.Browser-cfg-preload\" class=\"docClass\">preload</a>, which defines\nthe files that should be pre-loaded for each test. All urls in <a href=\"#!/api/Siesta.Project.Browser-cfg-preload\" rel=\"Siesta.Project.Browser-cfg-preload\" class=\"docClass\">preload</a> config\nare relative to the project html wrapper file <code>siesta.html</code>.</p>\n\n<p>You can add test files to the project using the <a href=\"#!/api/Siesta.Project.Browser-method-plan\" rel=\"Siesta.Project.Browser-method-plan\" class=\"docClass\">plan</a> method, which can be called several times.</p>\n\n<p>Note, that the project file contains regular javascript code, so you can configure it\ndifferently, depending on various conditions. Siesta includes <a href=\"https://github.com/ded/bowser\">Bowser</a>\nlibrary for browser detection, so you can for example mute some tests for certain browsers:</p>\n\n<pre><code>var isDev   = window.location.href.match(/localhost/);\nvar project = new <a href=\"#!/api/Siesta.Project.Browser\" rel=\"Siesta.Project.Browser\" class=\"docClass\">Siesta.Project.Browser</a>()\n\nproject.configure({\n    title       : 'Awesome Test Suite',\n\n    preload     : [\n        isDev ? '../ext-4.0.6/ext-all-debug.js' : '../ext-4.0.6/ext-all.js',\n        isDev ? '../yourproject-all-debug.js' : '../yourproject-all.js'\n    ]\n});\n\nproject.plan('some_test_for_all_browsers.t.js')\n\nif (!bowser.msie) project.plan('some_test_for_all_browsers_except_IE.t.js');\n\nproject.start()\n</code></pre>\n\n<p>Once the project is fully configured, you can launch it with the <a href=\"#!/api/Siesta.Project.Browser-method-start\" rel=\"Siesta.Project.Browser-method-start\" class=\"docClass\">start</a> method.</p>\n\n<p>As your test suite grows, you may need to start grouping your tests in a logical hierarchy. You can do this by passing special\n\"test file descriptor\" instead of the string to the <a href=\"#!/api/Siesta.Project.Browser-method-plan\" rel=\"Siesta.Project.Browser-method-plan\" class=\"docClass\">plan</a> method.\nThe descriptor should contain a group name and an array of child descriptors:</p>\n\n<pre><code>project.plan(\n    '011_single.t.js',\n    {\n        group       : 'Rendering tests',\n\n        items       : [\n            'rendering/grid.t.js',\n            'rendering/tree.t.js'\n        ]\n    }\n);\n</code></pre>\n\n<p>In turn, child descriptors can be groups as well. This feature is especially useful, when you need to override the\n<a href=\"#!/api/Siesta.Project.Browser\" rel=\"Siesta.Project.Browser\" class=\"docClass\">project</a> options for some group of tests (see the following section).</p>\n\n<p>See also <a href=\"#!/api/Siesta.Project.Browser-method-plan\" rel=\"Siesta.Project.Browser-method-plan\" class=\"docClass\">Siesta.Project.Browser.plan</a> method for additional information.</p>\n\n<h2 id='getting_started_sencha-section-configuring-the-individual-test'>Configuring the individual test</h2>\n\n<p>When configuring a \"test file descriptor\", one can also provide some of the project configuration options and they\nwill override the corresponding options that were provided to the project.\nThese options are explicitly marked in the <a href=\"#!/api/Siesta.Project.Browser\" rel=\"Siesta.Project.Browser\" class=\"docClass\">project</a> documentation.\nFor example, one can have a test with its own <code>preload</code> and <code>autoCheckGlobals</code> configs:</p>\n\n<pre><code>project.configure({\n    autoCheckGlobals    : false\n    ...\n})\n\nproject.start(\n    {\n        url     : '011_simple.t.js',\n\n        autoCheckGlobals : true,\n        preload : [\n            ...\n        ]\n    },\n    '012_complex.t.js'\n)\n</code></pre>\n\n<p>When specifying config options for a group descriptor, these options will override the configuration for all child descriptors of that group:</p>\n\n<pre><code>project.start(\n    {\n        group       : 'On demand loading',\n\n        // will override the `preload` option for all tests in this group\n        preload     : [\n            ...\n        ],\n\n        items       : [\n            ...\n        ]\n    }\n)\n</code></pre>\n\n<p>You can also provide test file descriptor in the test file itself, by adding it to the StartTest call:</p>\n\n<pre><code>StartTest({\n    autoCheckGlobals : false\n}, t =&gt; {\n    ...\n}) \n</code></pre>\n\n<p>Values from this object takes the highest priority and will override any other configuration.</p>\n\n<p>See also <a href=\"#!/api/Siesta.Project.Browser-method-plan\" rel=\"Siesta.Project.Browser-method-plan\" class=\"docClass\">Siesta.Project.Browser.plan</a> method for additional information.</p>\n\n<h2 id='getting_started_sencha-section-simulating-user-interaction'>Simulating user interaction</h2>\n\n<p>Siesta can simulate user interactions such as <a href=\"#!/api/Siesta.Test.Browser-method-click\" rel=\"Siesta.Test.Browser-method-click\" class=\"docClass\">click</a>, <a href=\"#!/api/Siesta.Test.Browser-method-doubleClick\" rel=\"Siesta.Test.Browser-method-doubleClick\" class=\"docClass\">double click</a>,\n<a href=\"#!/api/Siesta.Test.Browser-method-type\" rel=\"Siesta.Test.Browser-method-type\" class=\"docClass\">type</a>, <a href=\"#!/api/Siesta.Test.Browser-method-dragBy\" rel=\"Siesta.Test.Browser-method-dragBy\" class=\"docClass\">drag-drop</a> etc.</p>\n\n<p>Mouse simulating user actions are listed here: <a href=\"#!/api/Siesta.Test.UserAgent.Mouse\" rel=\"Siesta.Test.UserAgent.Mouse\" class=\"docClass\">Siesta.Test.UserAgent.Mouse</a>\nKeyboard simulating user actions are listed here: <a href=\"#!/api/Siesta.Test.UserAgent.Keyboard\" rel=\"Siesta.Test.UserAgent.Keyboard\" class=\"docClass\">Siesta.Test.UserAgent.Keyboard</a>\nTouch simulating user actions are listed here: <a href=\"#!/api/Siesta.Test.UserAgent.Touch\" rel=\"Siesta.Test.UserAgent.Touch\" class=\"docClass\">Siesta.Test.UserAgent.Touch</a></p>\n\n<p>By default Siesta uses synthetic events simulation, which is very fast.</p>\n\n<p>In some cases, however, you may need to test some intrinsic browser behavior which synthetic events can not reproduce.\nMost notable example will be testing the <code>:hover</code> CSS styles of some DOM elements.\nIn such case you can use native events simulation</p>\n\n<p>Keep in mind, for tests involving mouse interaction - you should not move the mouse during the execution of such test,\nto not interfere with simulation.</p>\n\n<h2 id='getting_started_sencha-section-user-actions-recorder'>User actions recorder</h2>\n\n<p>You can also record user actions as Siesta test. Please refer to this guide: <a href=\"#!/guide/event_recorder\">Using the event recorder.</a></p>\n\n<h2 id='getting_started_sencha-section-limitations-'>Limitations </h2>\n\n<p>Please note that the project page is limited by the same-origin policy as any other web page. You can include scripts/CSS from another domain, but XHR requests on older browsers\nare restricted to the same domain. The best approach is to run the test suite on the same domain as your application. You can also configure a server side proxy.</p>\n\n<h2 id='getting_started_sencha-section-writing-tests-in-bdd-style'>Writing tests in BDD style</h2>\n\n<p>Siesta supports BDD syntax for writing your tests. Please refer to this guide for more information: <a href=\"#!/guide/bdd_conventions\">Writing tests in BDD style.</a></p>\n\n<h2 id='getting_started_sencha-section-assertions'>Assertions</h2>\n\n<p>Some assertions are generic and cross platform - they belong to the <a href=\"#!/api/Siesta.Test\" rel=\"Siesta.Test\" class=\"docClass\">Siesta.Test</a> class. Others are browser/ExtJS specific - and those are\nlisted in the <a href=\"#!/api/Siesta.Test.Browser\" rel=\"Siesta.Test.Browser\" class=\"docClass\">Siesta.Test.Browser</a> / <a href=\"#!/api/Siesta.Test.ExtJS\" rel=\"Siesta.Test.ExtJS\" class=\"docClass\">Siesta.Test.ExtJS</a> classes.</p>\n\n<p>When an assertion passes - it shows a green checkmark with the assertion description (if description is not provided, Siesta will try to generate\na sensible default text). When it fails, it also tries to provide you with as much information about the failure as possible, including the\narguments passed to the assertion method. This is why you are encouraged to use various special assertions for each specific case. For example,\nlets say we would like to check if one value is greater than other. We could do that with a simple:</p>\n\n<pre><code>t.ok(value1 &gt; value2, 'Value1 is greater than value2')\n</code></pre>\n\n<p>But in case of a failure, the only additional information you will see will be something like:</p>\n\n<pre><code>Failed assertion [ok] at line xxx, \nGot  : false, \nNeed : \"truthy\" value\"\n</code></pre>\n\n<p>Compare with the output from the more specific \"isGreater\" assertion:</p>\n\n<pre><code>t.isGreater(value1, value2, 'Value1 is greater than value2')\n</code></pre>\n\n<p>It will output the message along with the provided arguments, instantly making it clear what happened:</p>\n\n<pre><code>Failed assertion [isGreater] at line xxx, \nGot                : value1, \nNeed, greater than : value2\n</code></pre>\n\n<p>So make sure you've scanned the documentation <a href=\"#!/api/Siesta.Test\" rel=\"Siesta.Test\" class=\"docClass\">Siesta.Test</a> or <a href=\"#!/api/Siesta.Test.ExtJS\" rel=\"Siesta.Test.ExtJS\" class=\"docClass\">Siesta.Test.ExtJS</a> - doing so will save you a lot of time.</p>\n\n<p>At this point you should have enough information to start - feel free to leave this document and start hacking your test suite together :)</p>\n\n<h2 id='getting_started_sencha-section-user-interface'>User interface</h2>\n\n<p>The project UI is stateful - it keeps the selected row, checked rows and options in cookies. You can refresh the project page anytime and your settings will remain.</p>\n\n<p>The rows in the test files grid has a context menu:\n<p><img src=\"guides/getting_started_sencha/images/ui-context-menu.png\" alt=\"autogen\" width=\"1600\" height=\"1200\"></p></p>\n\n<p>The \"Options\" button in the toolbar contains several checkboxes corresponding to project <a href=\"#!/api/Siesta.Project.Browser\" rel=\"Siesta.Project.Browser\" class=\"docClass\">configuration options</a>:\n<p><img src=\"guides/getting_started_sencha/images/ui-options-button.png\" alt=\"autogen\" width=\"1600\" height=\"1200\"></p></p>\n\n<h2 id='getting_started_sencha-section-testing-asynchronous-code'>Testing asynchronous code</h2>\n\n<p>When testing asynchronous code, the test function (passed to <code>StartTest</code>) may complete its execution, but the test itself\nwill still be running and incoming assertions will be valid. So, you need to explain to the test, that it needs to wait some time before finalizing.</p>\n\n<p>For that, before some asynchronous part of the code is started, indicate its beginning with <a href=\"#!/api/Siesta.Test-method-beginAsync\" rel=\"Siesta.Test-method-beginAsync\" class=\"docClass\">beginAsync</a> or\nwait methods. Then, once the async code complete, use <a href=\"#!/api/Siesta.Test-method-endAsync\" rel=\"Siesta.Test-method-endAsync\" class=\"docClass\">endAsync</a>\nor endWait accordingly.</p>\n\n<p>Both beginAsync and wait methods will start an \"asynchronous code frame\", the only difference is that <code>beginAsync</code> will return you an auto-generated id for it\n(which you need to save to some variable), and <code>wait</code> need to receive an unique id as parameter.</p>\n\n<p>For example:</p>\n\n<pre><code>var async = t.beginAsync();\n\nExt.Ajax.request({\n    url     : 'ajax_demo/sample.json',\n\n    success : function (response, opts) {\n        t.is(response, 'foobar', 'Response is correct');\n\n        t.endAsync(async);\n    },\n\n    failure : function (response, opts) {\n        t.fail(\"request failed\");\n\n        t.endAsync(async);\n    }\n});\n</code></pre>\n\n<p>Or the same with <code>wait</code>:</p>\n\n<pre><code>t.wait('xhrSample');\n\nExt.Ajax.request({\n    url     : 'ajax_demo/sample.json',\n\n    success : function (response, opts) {\n        t.is(response, 'foobar', 'Response is correct');\n\n        t.endWait('xhrSample');\n    },\n\n    failure : function (response, opts) {\n        t.fail(\"request failed\");\n\n        t.endWait('xhrSample');\n    }\n});\n</code></pre>\n\n<p>You can start as many asynchronous code \"frames\" as you need. By default, all frames will be forced to finalize after 15 seconds, so the whole test\nwill not get stuck in case of unexpected failures. You can configure this interval in the <a href=\"#!/api/Siesta.Test-method-beginAsync\" rel=\"Siesta.Test-method-beginAsync\" class=\"docClass\">beginAsync</a> and wait calls.</p>\n\n<p>Note, that you don't need to use these methods for built-in Siesta methods that accepts callbacks - Siesta already does that for you:</p>\n\n<pre><code>// the \"click\" method is asynchronous and accepts a callback\n// there's no need for `beginAsync` though, since its a built-in method\nt.click('.some-button', function () {\n}) \n</code></pre>\n\n<p>Additionally, if you return a <code>Promise</code> instance from the test function itself, Siesta will wait until that promise is resolved before finalizing the test.\nIn modern browsers, this allows us to use <code>async/await</code> functions:</p>\n\n<pre><code>StartTest(t =&gt; {\n    let someAsyncOperation = () =&gt; new Promise((resolve, reject) =&gt; {\n        setTimeout(() =&gt; resolve(true), 1000)\n    })\n\n    t.it('Doing async stuff', async t =&gt; {\n        let res = await someAsyncOperation()\n\n        t.ok(res, \"Async stuff finished correctly\")\n    })\n}) \n</code></pre>\n\n<h3 id='getting_started_sencha-section-chaining'>Chaining</h3>\n\n<p>Many methods of the test class are asynchronous and accept a callback. When you need to execute one such method after another,\nthe nesting level can grow quite quickly and affect the readability of the test code:</p>\n\n<pre><code>t.type(userNameField, 'username', function () {\n    t.type(passwordField, 'secret', function () {\n        t.click(loginButton, function () {\n            // done\n        })\n    })\n})\n</code></pre>\n\n<p>If you see such code in your tests, make sure you've checked the <a href=\"#!/api/Siesta.Test-method-chain\" rel=\"Siesta.Test-method-chain\" class=\"docClass\">chain</a> method. It allows you keep the nesting to a minimum:</p>\n\n<pre><code>t.chain(\n    function (next) {\n        t.type(userNameField, 'username', next)\n    },\n    function (next) {\n        t.type(passwordField, 'secret', next)\n    },\n    function (next) {\n        t.click(loginButton, next)\n    },\n    function () {\n        // done\n    }\n})\n</code></pre>\n\n<p>Or even more compact notation, providing action configuration objects instead of functions:</p>\n\n<pre><code>t.chain({\n    action  : 'type',\n    target  : userNameField, \n    text    : 'username'\n},{\n    action  : 'type',\n    target  : passwordField, \n    text    : 'secret'\n},{\n    action  : 'click',\n    target  : loginButton \n})\n</code></pre>\n\n<h3 id='getting_started_sencha-section-waiting'>Waiting</h3>\n\n<p>Since the nature of web applications is very asynchronous (Ajax calls, animations, etc.), be prepared to wait a lot in your tests.\nTo wait for a condition, do:</p>\n\n<pre><code>this.waitFor(\n    // The condition to check for, this line will mean waiting until the presence of #foo element\n    function() { return document.getElementById('foo'); },  \n\n    // The callback, after the condition has been fulfilled\n    function(el) { /* DO COOL STUFF */ }          \n);\n</code></pre>\n\n<p>You will find lots and lots of waitForXXX methods in the API to assist you in various situations, example:</p>\n\n<pre><code>t.waitForSelector('.some_class', function() {\n    // Found it\n});\n\ndocument.body.className = 'some_class'; // this fulfills the condition, \n                                        // and the callback function is called\n</code></pre>\n\n<p>You can also wait easily in a chain, by adding a \"waitFor\" step with a value equal to XXX of any waitForXXX command as the value:</p>\n\n<pre><code>t.chain({\n    { waitFor : 'selector', args : ['.some_class'] },   // calls waitForSelector which waits until \n                                                        // the some_class CSS class exists in the DOM\n\n    { waitFor : 500 },                                  // waits for 500 ms\n\n    { waitFor : 'elementVisible', args : [someEl] },    // calls waitForElementVisible and waits \n                                                        // for an element to become visible\n });\n</code></pre>\n\n<h2 id='getting_started_sencha-section-detecting-global-variable-leaks'>Detecting global variable leaks</h2>\n\n<p>Siesta has a special built-in assertion, called <a href=\"#!/api/Siesta.Test-method-verifyGlobals\" rel=\"Siesta.Test-method-verifyGlobals\" class=\"docClass\">t.verifyGlobals()</a>. It will scan the global properties of the test 'window' object,\nand compare them with the properties of a clean and fresh scope (properties from an empty <code>&lt;iframe&gt;</code>). In case it finds any \"unexpected\" globals\nit will report them as a test failure. You can specify your list of additional \"expected\" globals, using the <a href=\"#!/api/Siesta.Project.Browser-cfg-expectedGlobals\" rel=\"Siesta.Project.Browser-cfg-expectedGlobals\" class=\"docClass\">expectedGlobals</a> configuration\noption of the project, or by using the <a href=\"#!/api/Siesta.Test-method-expectGlobals\" rel=\"Siesta.Test-method-expectGlobals\" class=\"docClass\">t.expectGlobals</a> method of your test.</p>\n\n<p>You can enable this assertion to be executed automatically at the end of each test, by setting <a href=\"#!/api/Siesta.Project.Browser-cfg-autoCheckGlobals\" rel=\"Siesta.Project.Browser-cfg-autoCheckGlobals\" class=\"docClass\">autoCheckGlobals</a> to true on the project configuration.</p>\n\n<p>For example, in project:</p>\n\n<pre><code>project.configure({\n    autoCheckGlobals    : true,\n    expectedGlobals     : [ 'Ext', 'MyProject' ],\n    ...\n});\n</code></pre>\n\n<p>And then in tests:</p>\n\n<pre><code>// will suppress the complaints about these globals\nt.expectGlobals('Additional', 'Globals');\n</code></pre>\n\n<p>Please note however, that this feature is not supported in IE &lt;= 8, due to a bug in its JavaScript implementation: <a href=\"http://social.msdn.microsoft.com/Forums/en/iewebdevelopment/thread/065a5b93-6474-4756-aaea-1fd55d4a013a\">http://social.msdn.microsoft.com/Forums/en/iewebdevelopment/thread/065a5b93-6474-4756-aaea-1fd55d4a013a</a>\nAlso see: <a href=\"http://blogs.msdn.com/b/ericlippert/archive/2005/05/04/414684.aspx\">http://blogs.msdn.com/b/ericlippert/archive/2005/05/04/414684.aspx</a></p>\n\n<h2 id='getting_started_sencha-section-%22todo%22-assertions'>\"TODO\" assertions</h2>\n\n<p>Sometimes you might want to mark some assertions in a test as \"TODO\". For example, you might start to write a test covering an edge case,\nbut only fully implement it later. It's still desirable to run such assertions (sometimes they can accidentally start passing actually),\nbut if they fail - that doesn't mean a failure of the whole test.</p>\n\n<p>In such cases you can wrap your assertions using the <a href=\"#!/api/Siesta.Test-method-todo\" rel=\"Siesta.Test-method-todo\" class=\"docClass\">t.todo()</a> method:</p>\n\n<pre><code>t.todo('Scheduled for 4.1.x release', function (todo) {\n\n    var treePanel    = new Ext.tree.Panel()\n\n    todo.is(treePanel.getView().store, treePanel.store, \"NodeStore and TreeStore have been merged and there's only 1 store now\");\n})\n\n// or allow an assertion to fail until a certain date\nt.snooze(new Date(2016, 0, 1), function(t) {\n    // TODO fix this soon\n    t.expect(1).toBe(2);\n});\n</code></pre>\n\n<p>As you can see, the <code>todo</code> method accepts a description and a new test function. That function will receive a new \"TODO\" test instance,\nas the 1st argument (you can name it arbitrarily). Any failures coming from the methods of this new test instance\nwill be reported as \"green\", as they are expected.</p>\n\n<p>See also <a href=\"#!/api/Siesta.Test-method-snooze\" rel=\"Siesta.Test-method-snooze\" class=\"docClass\">snooze</a> method.</p>\n\n<h2 id='getting_started_sencha-section-extending-the-test-class'>Extending the test class</h2>\n\n<p>Quite often you may find yourself repeating various initialization code in your tests. Or you may have your own assertions,\nspecific to your data. To avoid repeation, you can extend the default Siesta test class by adding own methods to it.\nPlease refer to <a href=\"#!/guide/extending_test_class\">this guide</a> to know more.</p>\n\n<h2 id='getting_started_sencha-section-automation'>Automation</h2>\n\n<p>You can run your tests from the command line and generate various reports with test suite results.</p>\n\n<p>Please refer to <a href=\"#!/guide/siesta_automation\">Automation Guide.</a> for more information.</p>\n\n<h2 id='getting_started_sencha-section-best-practices'>Best practices</h2>\n\n<p>In general, you can organize your tests suite as you prefer - there are no strict rules where to place the files,\nhow to name them, etc. Choose the way that is more appropriate and efficient in your case.</p>\n\n<p>There are, however, some simple guidelines. First of all, treat your test suite as you treat your main codebase.\nAfter all <em>it is</em> a part of your codebase, just not visible to the end user. You will need to maintain it,\nupdate tests after refactoring (refactoring having a test suite is very pleasant experience btw) and so on.</p>\n\n<p>So, use the same coding style and practices as in your main codebase, provide meaningful assertion descriptions,\nwrite comments in non-trivial places, avoid repeating code and keep your test suite up-to-date.</p>\n\n<p>It is also a good idea to structure your tests into independent sections. Please refer to <a href=\"#!/guide/bdd_conventions\">BDD guide</a> for details</p>\n\n<h2 id='getting_started_sencha-section-advanced-setup'>Advanced setup</h2>\n\n<p>For own user interface, which is loaded on the project page, Siesta uses an optimized build of ExtJS, compiled with Sencha Cmd.\nOnly the classes, actualy used in the Siesta UI, are included in the \"siesta-all.js\" file. If you found that you need\nsome extra ExtJS class in your project, you can use alternative setup, as shown below.\nNote, that Siesta UI requires ExtJS version 6.0.1 precisely.</p>\n\n<pre><code>&lt;!DOCTYPE html&gt;\n&lt;html&gt;\n    &lt;head&gt;\n        &lt;meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\"&gt;\n        &lt;meta name=\"viewport\" content=\"width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no\"&gt;\n\n        &lt;title&gt;Sample project&lt;/title&gt;\n\n        &lt;link rel=\"stylesheet\" type=\"text/css\" href=\"__SIESTA_FOLDER__/resources/css/siesta-all.css\"&gt;\n\n        &lt;script type=\"text/javascript\" src=\"__EXTJS_6.0.1_FOLDER__/build/ext-all-debug.js'\"&gt;&lt;/script&gt;\n        &lt;script type=\"text/javascript\" src=\"__EXTJS_6.0.1_FOLDER__/build/classic/theme-triton/theme-triton.js\"&gt;&lt;/script&gt;\n        &lt;script type=\"text/javascript\" src=\"__SIESTA_FOLDER__/siesta-no-ext-all.js\"&gt;&lt;/script&gt;\n\n        &lt;script type=\"text/javascript\" src=\"index.js\"&gt;&lt;/script&gt;\n    &lt;/head&gt;\n\n    &lt;body&gt;\n    &lt;/body&gt;\n&lt;/html&gt;\n</code></pre>\n\n<h2 id='getting_started_sencha-section-running-tests-in-nodejs'>Running tests in NodeJS</h2>\n\n<p>If your code is written in a cross-platform manner, you can also run your tests in NodeJS. For that, first of all, install Siesta in the \"node_modules\"\nfolder of your project. Then, in the project, detect the platform and choose a correct project class, like this:</p>\n\n<pre><code>var Harness\n\nif (typeof process != 'undefined' &amp;&amp; process.pid) {\n    Harness = require('siesta');\n} else {\n    Harness = new <a href=\"#!/api/Siesta.Project.Browser\" rel=\"Siesta.Project.Browser\" class=\"docClass\">Siesta.Project.Browser</a>()\n}\n</code></pre>\n\n<p>And finally, launch the project file in NodeJS:</p>\n\n<pre><code>&gt; node tests/index.js\n</code></pre>\n\n<p>You will see the same output as when running the test suite in Puppeteer. See also the \"nodejs-browsers\" example in the \"/examples\" folder.</p>\n\n<p>In case of any failures in the test suite, the command will exit with non-zero exit code.</p>\n\n<h2 id='getting_started_sencha-section-buy-this-product'>Buy this product</h2>\n\n<p>Visit our store: <a href=\"https://bryntum.com/store/siesta\">https://bryntum.com/store/siesta</a></p>\n\n<h2 id='getting_started_sencha-section-support'>Support</h2>\n\n<p>Ask question in our community forum: <a href=\"https://www.bryntum.com/forum/viewforum.php?f=20\">https://www.bryntum.com/forum/viewforum.php?f=20</a></p>\n\n<p>Subscribers can post expedited questions in Premium Forum: <a href=\"https://www.bryntum.com/forum/viewforum.php?f=21\">https://www.bryntum.com/forum/viewforum.php?f=21</a></p>\n\n<p>Please report any bugs through the web interface at <a href=\"https://www.assembla.com/spaces/bryntum/support/tickets\">https://www.assembla.com/spaces/bryntum/support/tickets</a></p>\n\n<h2 id='getting_started_sencha-section-see-also'>See also</h2>\n\n<p>Siesta web-page: <a href=\"https://bryntum.com/products/siesta\">https://bryntum.com/products/siesta</a></p>\n\n<p>Other Bryntum products: <a href=\"https://bryntum.com/products\">https://bryntum.com/products</a></p>\n\n<h2 id='getting_started_sencha-section-attribution'>Attribution</h2>\n\n<p>This software contains icons from the following icon packs (licensed under Creative Common 2.5/3.0 Attribution licenses)</p>\n\n<ul>\n<li><a href=\"http://www.famfamfam.com/lab/icons/silk/\">http://www.famfamfam.com/lab/icons/silk/</a></li>\n<li><a href=\"http://led24.de/iconset/\">http://led24.de/iconset/</a></li>\n<li><a href=\"http://p.yusukekamiyamane.com/\">http://p.yusukekamiyamane.com/</a></li>\n<li><a href=\"http://rrze-icon-set.berlios.de/index.html\">http://rrze-icon-set.berlios.de/index.html</a></li>\n<li><a href=\"http://www.smashingmagazine.com/2009/05/20/flavour-extended-the-ultimate-icon-set-for-web-designers/\">http://www.smashingmagazine.com/2009/05/20/flavour-extended-the-ultimate-icon-set-for-web-designers/</a></li>\n<li><a href=\"http://www.doublejdesign.co.uk/products-page/icons/super-mono-icons/\">http://www.doublejdesign.co.uk/products-page/icons/super-mono-icons/</a></li>\n<li><a href=\"http://pixel-mixer.com/\">http://pixel-mixer.com/</a></li>\n</ul>\n\n\n<p>Thanks a lot to the authors of the respective icons packs.</p>\n\n<h2 id='getting_started_sencha-section-copyright-and-license'>COPYRIGHT AND LICENSE</h2>\n\n<p>Copyright (c) 2009-2018, Bryntum &amp; Nickolay Platonov</p>\n\n<p>All rights reserved.</p>\n","title":"Sencha environment"});