Ext.data.JsonP.siesta_launchers({"guide":"<h2 id='siesta_launchers-section-intro'>Intro</h2>\n<div class='toc'>\n<p><strong>Contents</strong></p>\n<ol>\n<li><a href='#!/guide/siesta_launchers-section-intro'>Intro</a></li>\n<li><a href='#!/guide/siesta_launchers-section-running-tests-using-selenium-webdriver'>Running tests using Selenium WebDriver</a></li>\n<li><a href='#!/guide/siesta_launchers-section-running-tests-using-puppeteer'>Running tests using Puppeteer</a></li>\n<li><a href='#!/guide/siesta_launchers-section-running-tests-using-remote-selenium-webdriver-server'>Running tests using remote Selenium WebDriver server</a></li>\n<li><a href='#!/guide/siesta_launchers-section-configuration-file'>Configuration file</a></li>\n<li><a href='#!/guide/siesta_launchers-section-native-events'>Native events</a></li>\n<li><a href='#!/guide/siesta_launchers-section-parallelization'>Parallelization</a></li>\n<li><a href='#!/guide/siesta_launchers-section-reports-'>Reports </a></li>\n<li><a href='#!/guide/siesta_launchers-section-exit-codes'>Exit codes</a></li>\n<li><a href='#!/guide/siesta_launchers-section-buy-this-product'>Buy this product</a></li>\n<li><a href='#!/guide/siesta_launchers-section-support'>Support</a></li>\n<li><a href='#!/guide/siesta_launchers-section-see-also'>See also</a></li>\n<li><a href='#!/guide/siesta_launchers-section-copyright-and-license'>COPYRIGHT AND LICENSE</a></li>\n</ol>\n</div>\n\n<p>Running the test suite manually in browsers is very convenient as it allows you to easily debug your tests. However, when\nsetting up a continuous integration process, it quickly becomes time-consuming - ideally you should to run the test suite\nin each and every browser after (or before) each and every commit.</p>\n\n<p>This guide describes how you can automate the launching of a Siesta test suite.</p>\n\n<p><strong>Please note:</strong> This functionality is only available in the Siesta Standard package.</p>\n\n<h2 id='siesta_launchers-section-running-tests-using-selenium-webdriver'>Running tests using Selenium WebDriver</h2>\n\n<p><strong>Important</strong>. Using the WebDriver requires some manual configuration steps for IE and Safari. Please refer to these\npages <a href=\"https://github.com/SeleniumHQ/selenium/wiki/InternetExplorerDriver\">https://github.com/SeleniumHQ/selenium/wiki/InternetExplorerDriver</a> and <a href=\"https://github.com/SeleniumHQ/selenium/wiki/SafariDriver\">https://github.com/SeleniumHQ/selenium/wiki/SafariDriver</a>.\nAlso, when running test suite in IE, make sure the IE window is focused (on top of other windows) and mouse cursor is outside of the IE window.</p>\n\n<p>You can automate the launching of your test suite in several browsers, using Selenium WebDriver.\nWebDriver requires the presence of actual browsers in the operation system, installed in the default locations.</p>\n\n<p>For browsers that supports headless mode (Chrome and Firefox) you can use <code>--headless</code> option to prevent opening of browser window (and interfering\nwith your normal work).</p>\n\n<p>On MacOS and Linux:</p>\n\n<pre><code>&gt; __SIESTA__/bin/webdriver http://yourproject/tests/index.html [OPTIONS]\n</code></pre>\n\n<p>On Windows:</p>\n\n<pre><code>&gt; __SIESTA__\\bin\\webdriver http://yourproject/tests/index.html [OPTIONS]\n</code></pre>\n\n<p>Here, the <code>__SIESTA__</code> placeholder is the path to your Siesta package. The launcher script <code>bin/webdriver</code> accepts the URL to\nyour html wrapper for the Siesta project (<code>index.html</code>) and several optional options.</p>\n\n<p>All options should start with double minus, e.g: <code>--browser chrome</code> or <code>--browser=chrome</code>. Please see the output of the <code>bin/webdriver --help</code> for\nall available options. Most important options are below:</p>\n\n<ul>\n<li><code>help</code> - prints help message with all available options</li>\n<li><code>browser browsername</code> - can be one of \"firefox / chrome / ie / safari / edge\"'.</li>\n<li><code>headless</code> - launch the browser headless (not visible and not interfering with the current desktop).</li>\n<li><code>max-workers</code> - maximum number of parallel testing \"threads\" that can be opened simultaneously. Please refer to a \"Parallelization\" section below.</li>\n<li><code>include regexp</code>      - a regexp to filter the urls of tests. When provided, only the tests with urls matching this filter be executed</li>\n<li><code>verbose</code> - will include the information about every individual assertions to the output. By default, only failed assertions will be shown</li>\n<li><code>debug</code> - will enable various debugging messages</li>\n<li><code>report-format</code> - the format of the report, see the \"Reporting the results of a test suite execution in a structured format\" section below</li>\n<li><code>report-file</code> - the file to save the report to</li>\n</ul>\n\n\n<p>You can also use the <code>--cap</code> switch to specify various Selenium capabilities for example \"--cap browserName=firefox --cap platform=XP\",\nsee <a href=\"https://code.google.com/p/selenium/wiki/DesiredCapabilities\">https://code.google.com/p/selenium/wiki/DesiredCapabilities</a></p>\n\n<p>In case of any failures in the test suite the command will exit with non-zero exit code. See \"Exit codes\" section for details.</p>\n\n<h2 id='siesta_launchers-section-running-tests-using-puppeteer'>Running tests using Puppeteer</h2>\n\n<p>Puppeteer is an automation library for the headless Chrome browser. Puppeteer uses real rendering engine of the Chromium browser -\nyou can trust the results from Puppeteer as if they were received from \"real\" Chrome.</p>\n\n<p>Puppeteer will download a required Chrome version on the 1st launch. Further launches will be instant.</p>\n\n<p>On MacOS and Linux:</p>\n\n<pre><code>&gt; __SIESTA__/bin/puppeteer http://yourproject/tests/index.html [OPTIONS]\n</code></pre>\n\n<p>On Windows:</p>\n\n<pre><code>&gt; __SIESTA__\\bin\\puppeteer http://yourproject/tests/index.html [OPTIONS]\n</code></pre>\n\n<p>Here, the <code>__SIESTA__</code> placeholder is the path to your Siesta package. The launcher script <code>bin/puppeteer</code> accepts the URL to your html wrapper for the Siesta project (<code>index.html</code>)\nand several optional options.</p>\n\n<p>Puppeteer supports almost the same set of options as the webdriver launcher, please refer to: <code>bin/puppeteer --help</code></p>\n\n<h2 id='siesta_launchers-section-running-tests-using-remote-selenium-webdriver-server'>Running tests using remote Selenium WebDriver server</h2>\n\n<p>You can launch the test suite (and receive the results) on one machine, but physically open the browsers on some other machine using RemoteWebDriver server.\nTo do that, first start the RemoteWebDriver server on target machine:</p>\n\n<p>On MacOS and Linux:</p>\n\n<pre><code>&gt; __SIESTA__/bin/webdriver-server [OPTIONS]\n</code></pre>\n\n<p>On Windows:</p>\n\n<pre><code>&gt; __SIESTA__\\bin\\webdriver-server [OPTIONS]\n</code></pre>\n\n<p>\"webdriver-server\" is a very thin wrapper around \"selenium-server-standalone-xx.jar\" which just specifies the location of binaries for various browsers. It bypass\nany command line options to that jar file. For example, to specify the port of server (default value is 4444), specify it as:</p>\n\n<pre><code>&gt; __SIESTA__/bin/webdriver-server -port 4444\n</code></pre>\n\n<p>For a list of available options for server, launch it with \"-help\" switch:</p>\n\n<pre><code>&gt; __SIESTA__/bin/webdriver-server -help\n</code></pre>\n\n<p>Also, please refer to: <a href=\"https://github.com/SeleniumHQ/selenium/wiki/RemoteWebDriverServer\">https://github.com/SeleniumHQ/selenium/wiki/RemoteWebDriverServer</a></p>\n\n<p>Then, when launching the test suite with \"webdriver\" launcher, specify an additional \"--host\" option:</p>\n\n<pre><code>&gt; __SIESTA__/bin/webdriver http://my.project.url/tests --host remote.webdriver.host --port 4444\n</code></pre>\n\n<p>You can use the \"--cap\" switch to specify various Selenium capabilities for example \"--cap browserName=firefox --cap platform=XP\",\nsee <a href=\"https://code.google.com/p/selenium/wiki/DesiredCapabilities\">https://code.google.com/p/selenium/wiki/DesiredCapabilities</a></p>\n\n<h2 id='siesta_launchers-section-configuration-file'>Configuration file</h2>\n\n<p>Configuration file can be specified with the \"--config-file\" command line option. If this option is not given, Siesta will\nautomatically check for the presence of \"siesta.json\" file in the current directory or one of the parent directories.\nIf such file presents it will be read.</p>\n\n<p>Config file should contain JSON, with a bit less strict formatting rules. Notably, the // and /<em> </em>/ comments are allowed,\nidentifiers can be unquoted, strings can be quoted with single-quote character.</p>\n\n<p>The format of config file:</p>\n\n<pre><code>{\n    // entries corresponds to project configs and overrides \n    // the values from the \"project.configure()\" call\n    project  : {\n        transparentEx       : true\n    },\n\n    // enrties corresponds to command line options, but *not* overrides them - \n    // options from the command line takes precendence\n    cmd : {\n        rerun-failed        : true\n    }\n}\n</code></pre>\n\n<p>Command line options like <code>debug</code> or <code>verbose</code> should be specified with the boolean value <code>true/false</code>.\nIf command line option is repeatable, then you can specify an array with several values.</p>\n\n<h2 id='siesta_launchers-section-native-events'>Native events</h2>\n\n<p>By default Siesta uses synthetic events simulation, which is very fast. However, in some cases, you may need to test some intrinsic browser\nbehavior which synthetic events can not reproduce. Most notable example will be testing the <code>:hover</code> CSS styles of some DOM elements.\nIn such case you can use native events simulation</p>\n\n<h2 id='siesta_launchers-section-parallelization'>Parallelization</h2>\n\n<p>Siesta allows you to run the test suite in several parallel \"threads\" or \"workers\", speeding up the execution almost linearly by the\nnumber of \"workers\".</p>\n\n<p>This can be done using the <code>--max-workers</code> option, which specifies the maximum number of \"workers\".</p>\n\n<p>There are some caveats though, related to focus handling. Even if several browser windows are opened simultaneously,\nthe focus can remain only in one of them, and all browsers \"fights\" for it (focus jumps randomly from one window to\nanother). This naturally leads to tests instability, since code often relies on some element being focused.\nSo, in general, the <code>--max-workers</code> can be used safely only for the non-visual tests. However see below.</p>\n\n<ul>\n<li>Cloud providers</li>\n</ul>\n\n\n<p>When using cloud-based infrastructure (SauceLabs or BrowserStack), each test page is running inside of the own VM,\nwhich guarantees the exclusive focus owning and makes it safe to use <code>--max-workers</code>.</p>\n\n<ul>\n<li>XVFB</li>\n</ul>\n\n\n<p>Linux only. When using Linux with the \"Xvfb\" package installed, one can add special command-line switch <code>--xvfb</code>, which launch\nevery browser window inside the separate virtual X desktop (with own focus handling). This makes it safe to specify\n<code>--max-workers</code> more than 1 and speeds up the test suite execution by the number of workers! For this option to work,\nthe <code>Xvfb</code> is expected to be in PATH, same for the browser binaries, <code>google-chrome-stable</code> and <code>firefox</code>. This feature is\nnow deprecated in favor of headless mode (which does essentially the same thing)</p>\n\n<ul>\n<li>Headless</li>\n</ul>\n\n\n<p>The headless mode of the webdriver launcher provides an isolated environment for every browser window as well. It is usually safe\nto use several workers in this case. Note, however, that currently headless Chrome is known to be very slow on Windows</p>\n\n<p><strong>Important</strong>. When value of <code>--max-workers</code> is more than 1, the order of tests execution is not defined. A test, that goes lower\nin the <code>project.start()</code> list, can be executed before the test above it. This is simply because all tests are divided in several\n\"threads\" and all threads are executed simultaneously. You should not rely on some test being run after another, instead,\nevery test should execute standalone (allocate exclusive resources, perform all necessary setup).</p>\n\n<h2 id='siesta_launchers-section-reports-'>Reports </h2>\n\n<p>You can export the results of a test suite execution in the structured format. To do that, provide the <code>--report-format</code> and <code>--report-file</code>\noptions to the launcher. Supported formats are \"HTML\", \"JSON\", \"JSONS\" and \"JUnit\" (case insensitive). These options can be repeated\nseveral times, to generate several reports.</p>\n\n<p>The <code>JSON</code> and <code>JSONS</code> formats are almost identical, the <code>JSON</code> format contains the test results as a flat list,\nthe <code>JSONS</code> contains results as a tree structure, based on the test groups information.</p>\n\n<p><code>HTML</code> report is basically a <code>JSONS</code> report, plus visualization files. It can be used to examine test suite execution results in the browser.\nWhen using <code>HTML</code> report, the <code>--report-file</code> option actually specifies the <em>directory</em> for files (as this report consists from several files).\nThe <code>JSONS</code> report will be created as <code>report-data.json</code> file in that directory.</p>\n\n<p>An example:</p>\n\n<pre><code>siesta/bin/webdriver http://localhost/YourApplication/tests --report-format JSON --report-file foo.json\n</code></pre>\n\n<p>And this will generate a file named <code>foo.json</code> containing the following JSON structure:</p>\n\n<pre><code>{\n    \"testSuiteName\" : \"Siesta self-hosting test suite\",\n    \"startDate\"     : 1343114314723,\n    \"endDate\"       : 1343114315401,\n    \"passed\"        : true,\n    \"testCases\"     : [{\n        \"url\"           : \"010_sanity.t.js\",\n        \"startDate\"     : 1343114315390,\n        \"endDate\"       : 1343114315396,\n        \"passed\"        : true,\n        \"assertions\"    : [{\n            \"passed\"        : true,\n            \"description\"   : \"Siesta is here\",\n            \"type\"          : \"Siesta.Result.Assertion\"\n        }, {\n            \"passed\"        : true,\n            \"description\"   : \"<a href=\"#!/api/Siesta.Test\" rel=\"Siesta.Test\" class=\"docClass\">Siesta.Test</a> is here\",\n            \"type\"          : \"Siesta.Result.Assertion\"\n        }, {\n            \"passed\"        : true,\n            \"description\"   : \"<a href=\"#!/api/Siesta.Project\" rel=\"Siesta.Project\" class=\"docClass\">Siesta.Project</a> is here\",\n            \"type\"          : \"Siesta.Result.Assertion\"\n        }, {\n            \"passed\"        : false,\n            \"description\"   : \"Field 1 focused\",\n            \"annotation\"    : \"Failed assertion `ok` at line 27 of keyevents/050_tab_key_focus2.t.js\\nGot: false\\nNeed \\\"truthy\\\" value\",\n            \"sourceLine\"    : \"27\",\n            \"name\"          : \"ok\"\n            \"type\"          : \"Siesta.Result.Assertion\"\n        }]\n    }]\n}\n</code></pre>\n\n<p>Using the JUnit report option looks like this:</p>\n\n<pre><code>siesta/bin/webdriver http://localhost/YourApplication/tests --report-format junit --report-file foo.xml\n</code></pre>\n\n<p>This will generate a file named <code>foo.xml</code> containing the following JUnit XML structure:</p>\n\n<pre><code>&lt;testsuite errors=\"0\" failures=\"1\" hostname=\"localhost:8085\" name=\"Ext Scheduler Test Suite\" tests=\"2\" time=\"3.594\" timestamp=\"2012-06-06T08:55:21.520\"&gt;\n\n    &lt;testcase classname=\"Bryntum.Test\" name=\"lifecycle/040_schedulergrid.t.js\" time=\"1.238\"&gt;\n        &lt;failure message=\"Oops\" type=\"FAIL\"&gt;&lt;/failure&gt;\n    &lt;/testcase&gt;\n\n    &lt;testcase classname=\"Bryntum.Test\" name=\"lifecycle/042_schedulergrid_right_columns.t.js\" time=\"0.818\"&gt;&lt;/testcase&gt;\n&lt;/testsuite&gt;\n</code></pre>\n\n<h2 id='siesta_launchers-section-exit-codes'>Exit codes</h2>\n\n<ul>\n<li>0 - All tests passed successfully</li>\n<li>1 - Some tests failed</li>\n<li>3 - No supported browsers available on this machine</li>\n<li>4 - No tests to run (probably filter doesn't match any test url)</li>\n<li>5 - Can't open project page</li>\n<li>6 - Wrong command line arguments</li>\n<li>8 - Exit after showing the Siesta version (when <code>--version</code> is provided on the command line)</li>\n</ul>\n\n\n<h2 id='siesta_launchers-section-buy-this-product'>Buy this product</h2>\n\n<p>Visit our store: <a href=\"https://bryntum.com/store/siesta\">https://bryntum.com/store/siesta</a></p>\n\n<h2 id='siesta_launchers-section-support'>Support</h2>\n\n<p>Ask a question in our community forum: <a href=\"https://www.bryntum.com/forum/viewforum.php?f=20\">https://www.bryntum.com/forum/viewforum.php?f=20</a></p>\n\n<p>Subscribers can post expedited questions in Premium Forum: <a href=\"https://www.bryntum.com/forum/viewforum.php?f=21\">https://www.bryntum.com/forum/viewforum.php?f=21</a></p>\n\n<p>Please report any bugs through the web interface at <a href=\"https://www.assembla.com/spaces/bryntum/support/tickets\">https://www.assembla.com/spaces/bryntum/support/tickets</a></p>\n\n<h2 id='siesta_launchers-section-see-also'>See also</h2>\n\n<p>Siesta web-page: <a href=\"https://bryntum.com/products/siesta\">https://bryntum.com/products/siesta</a></p>\n\n<p>Other Bryntum products: <a href=\"https://bryntum.com/products\">https://bryntum.com/products</a></p>\n\n<h2 id='siesta_launchers-section-copyright-and-license'>COPYRIGHT AND LICENSE</h2>\n\n<p>Copyright (c) 2009-2018, Bryntum AB &amp; Nickolay Platonov</p>\n\n<p>All rights reserved.</p>\n","title":"Siesta launchers & reports"});